#!/usr/bin/env python
# encoding: utf-8

import os
import re
import codecs
import time
import cgi
import urllib
import xml.sax.saxutils

import cgitb; cgitb.enable()

SiteName = u'unitas'
ScriptName = os.path.basename(os.environ.get('SCRIPT_NAME', ''))
TabWidth = 4

SnippetA = u'''\
%s<br />
<form class="edit" action="%s" method="post">
 <input type="hidden" name="page" value="%s" />
 <textarea id="txt" name="txt" rows="20"></textarea><br />
 <input type="submit" id="write" value=" new " />
</form>
'''

SnippetB = u'''\
%s
<div id="menu"><a href="" id="show">&gt;&gt; edit</a></div>
<div id="view">%s</div>
<form id="edit" action="%s" method="post">
  <input type="hidden" name="page" value="%s" />
<textarea id="txt" name="txt" rows="%d">
%s</textarea><br />
  <input type="submit" id="write" value=" write " />
  <input type="checkbox" name="update" value="on" />update
</form>
<div id="foot">%s</div>
'''

SnippetC = u'''\
Content-type: text/html; charset=utf-8

<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja" lang="ja">
<head>
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
  <meta http-equiv="Content-Script-Type" content="text/javascript" />
  <meta http-equiv="Content-Style-Type" content="text/css" />
  <title>%s</title>
  <link rel="stylesheet" type="text/css" href="style.css" />
  <script type="text/javascript" src="jquery-latest.pack.js"></script>
  <script type="text/javascript" src="script.js"></script>
</head>
<body>
<div id="all">
<div id="page">%s</div>
<div id="side">
  <a href="" id="newa">new</a><br />
  <form id="newf" action="%s" method="get">
  <input type="text" class="txt" size="20" name="page" value="" />
  </form>
  <a href="" id="scha">search</a><br />
  <form id="schf" action="%s" method="get">
  <input type="text" class="txt" size="20" name="word" value="" />
  </form>
  <p id="list">%s</p>
</div>
</div>
</body>
</html>
'''

icgi = cgi.FieldStorage(keep_blank_values=True)

def cgip(key):
    value = icgi.getfirst(str(key))
    if value: value = value.decode('utf-8')
    return value

def esc(s):
    return xml.sax.saxutils.escape(s) if s or s == '' else None
    
def unesc(s):
    return xml.sax.saxutils.unescape(s) if s or s == '' else None
    
def files():
    sortkey = lambda x: float(os.path.getmtime(os.path.join('d', x)))
    entries = os.listdir('d')
    entries.sort(key=sortkey, reverse=True)
    return entries

def link_self(pagename):
    if pagename == 'FrontPage':
        link = ''
    else:
        link = '?%s' % esc(pagename)
    page = esc(urllib.unquote(pagename).decode('utf-8'))

    return '<a href="%s%s">%s</a>' % (ScriptName, link, page)

SpecialCharsRegex = re.compile(r'''
    (
    \n|\t|\s|
    \[\[.+?\]\]|
    (http:|https:)\/\/\S+
    )
''', re.VERBOSE)
WikiLinkRegex = re.compile(r'\[\[(\S+?)\]\]')
HtmlLinkRegex = re.compile(r'\[\[((http:|https)\S+?)\s+?(.+?)\]\]')
UrlLinkRegex = re.compile(r'((http:|https:)\/\/\S+)')

def _substring(m):
    s = m.group(1)

    if s == '\n':
        return '<br />\n'
    elif s == '\t':
        return '&nbsp;' * TabWidth
    elif s == ' ':
        return '&nbsp;'
    elif UrlLinkRegex.match(s):
        return '<a href="%s">%s</a>' % (s, s)

    mo = WikiLinkRegex.match(s)
    if mo:
        g = mo.group(1)
        pagehref = urllib.quote(unesc(g).encode('utf-8'))
        return '<a href="?%s">%s</a>' % (pagehref, g)

    mo = HtmlLinkRegex.match(s)
    if mo:
        g = mo.groups()
        return '<a href="%s">%s</a>' % (g[0], g[2])

    return s

def s2view(s):
    return SpecialCharsRegex.sub(_substring, s)

def xtitle(lst):
    base = ['<a href="%s">%s</a>' % (ScriptName, SiteName)]
    title = ' - '.join([x for x in (base + lst) if x])
    return '<h1>%s</h1>' % title

def xpage(t):
    def _fread(fn):
        fh = codecs.open('d/%s' % fn, encoding='utf-8')
        page = fh.read()
        fh.close()
        return page
        
    if cgip('word'):
        content = list()
        _files = files()

        content.append('%s<p>' % xtitle([esc(t), 'title']))
        for fn in _files:
            if t in fn:
                content.append('%s<br />' % link_self(fn))
        content.append('</p>')

        content.append('%s<p>' % xtitle([esc(t), 'text']))
        for fn in _files:
            for line in _fread(fn).split("\n"):
                if t in line:
                    page = '<p>%s<br />%s</p>' % (link_self(fn), esc(line))
                    content.append(page)
        content.append('</p>')

        return ''.join(content)

    path = t if t else 'FrontPage'
    path = 'd/%s' % urllib.quote(path.encode('utf-8'))
    title = xtitle([esc(t)])
    pagetitle = t if t else 'FrontPage'

    if os.path.exists(path) and os.path.isfile(path):
        fh = codecs.open(path, encoding='utf-8')
        page = fh.read()
        fh.close()

        html = s2view(page)
        rows = page.count('\n') * 1.2 + 10
        plaintext = esc(page.replace('\r\n', '\n'))
        _lastmod = time.localtime(os.path.getmtime(path))
        lastmod = time.strftime('%Y/%m/%d %H:%M', _lastmod)

        return SnippetB % (title, html, ScriptName,
                           pagetitle, rows, plaintext, lastmod)
    else:
        return SnippetA % (title, ScriptName, pagetitle)

def redirect(to):
    if to:
        to = urllib.quote(to.encode('utf-8'))
    print 'Location: %s\n\n\n' % '?'.join([x for x in [ScriptName, to] if x])

    return

def main():
    if cgip('txt') or cgip('txt') == '':
        pname = cgip('page')
        path = 'd/%s' % urllib.quote(pname.encode('utf-8'))

        if cgip('txt') == '' and os.path.isfile(path):
            os.unlink(path)
        else:
            if os.path.isfile(path) and not cgip('update'):
                u = os.path.getmtime(path)
            else:
                u = None

            inpt = cgip('txt').replace('\r\n', '\n')
            fh = codecs.open(path, 'w', encoding='utf-8')
            fh.write(inpt)
            fh.close()

            if u:
                os.utime(path, (u, u))
            os.chmod(path, 0600)

        pagename = cgip('page')
        pagename = pagename if pagename != 'FrontPage' else None
        return redirect(pagename)
    else:
        t = cgip('page') or cgip('word') or None
        if not t:
            keys = icgi.keys()
            t = keys[0].decode('utf-8') if keys else ''

        title = ' - '.join([x for x in [SiteName, t] if x])
        html = xpage(t)
        titles = '<br />'.join([link_self(i) for i in files()])

        content = SnippetC % (title, html, ScriptName, ScriptName, titles)
        print content.encode('utf-8')
        return

main()
